﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;
using System.Diagnostics;

namespace SOAP_CustomFields
{
    class CustomFields
    {
        enum TestConfigType
        {
            eTestConfig_User = 1,
            eTestConfig_Standard = 2
        }

        static void Main(string[] args)
        {
            // This will stream all output to a text file
            StringWriter log_output = null;
            try
            {
               log_output = new StringWriter();
            }
            catch (IOException e)
            {
               Console.WriteLine("{0} - Cannot create log output.", e.Message);
               return;
            }

            Console.SetOut(log_output);

            ttsoapcgi svr = null;
            long cookieSrc = 0, cookieDest = 0;

            try
            {
                // Get the ID to replicate
                long id = long.Parse(Environment.GetEnvironmentVariable("RECORDID"));

                svr = new ttsoapcgi();

                // Log into both the source and destination projects
                CProject sourceProject = new CProject();
                sourceProject.database = new CDatabase();
                sourceProject.database.name = "Source Project";
                sourceProject.options = new CProjectDataOption[1];
                sourceProject.options[0] = new CProjectDataOption();
                sourceProject.options[0].name = "TestTrack Pro";

                CProject destinationProject = new CProject();
                destinationProject.database = new CDatabase();
                destinationProject.database.name = "Destination Project";
                destinationProject.options = new CProjectDataOption[1];
                destinationProject.options[0] = new CProjectDataOption();
                destinationProject.options[0].name = "TestTrack Pro";

                cookieSrc = svr.ProjectLogon(sourceProject, "SOAPUser", "myPassword");
                cookieDest = svr.ProjectLogon(destinationProject, "SOAPUser", "myPassword");

                CDefect dft = svr.getDefectByRecordID(cookieSrc, id, true);

                // Check that inherent dropdown values are defined in the destination project
                CheckInherentDropdownFields(svr, cookieDest, dft);

                // Check our custom fields
                PopulateCustomFields(dft);

                // Clear the record ID and event list
                dft.recordid = 0;
                dft.eventlist = null;
                Console.WriteLine("Defect copied to Destination Project with record ID {0}", 
                   svr.addDefect(cookieDest, dft));
            }
            catch (SoapException e)
            {
                Console.WriteLine("SOAP Fault: {0} - {1}", e.Detail.InnerText, e.Message);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            finally
            {
                try
                {
                    if (svr != null)
                    {
                        svr.DatabaseLogoff(cookieSrc);
                        svr.DatabaseLogoff(cookieDest);
                    }
                }
                catch (SoapException e)
                {
                    Console.WriteLine("SOAP Fault: {0} - {1}", e.Detail.InnerText, e.Message);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                }
            }

            log_output.Close();

            string sOutput = log_output.ToString();
            string sSource = "TestTrack Custom Field Demo";

            if (sOutput.Length > 0)
            {
               if (!EventLog.SourceExists(sSource))
                  EventLog.CreateEventSource(sSource, "Application");

               EventLog.WriteEntry(sSource, log_output.ToString());
            }
        }

       private static void CheckInherentDropdownFields(ttsoapcgi svr, long cookie, CDefect dft)
       {
          // Product
          CheckInherentDropdownField(svr, cookie, "Product", dft.product);
          
          // Type
          CheckInherentDropdownField(svr, cookie, "Type", dft.type);
          
          // Priority has been renamed Urgency
          CheckInherentDropdownField(svr, cookie, "Urgency", dft.priority);
          
          // Component
          CheckInherentDropdownField(svr, cookie, "Component", dft.component);
          
          // Severity
          CheckInherentDropdownField(svr, cookie, "Severity", dft.severity);
          
          // Disposition
          CheckInherentDropdownField(svr, cookie, "Disposition", dft.disposition);
          
          // Entered By
          if (dft.enteredby != null)
          {
             if (!ArrayContainsValue(GetFieldValues(svr, cookie, "Entered by"), dft.enteredby))
             {
                // Reset the entered by to this user
                dft.enteredby = "User, SOAP";
             }
          }
          
          // These fields are located on the reported by record. There may be multiple reported by records so check each one
          
          // Reproduced
          CFieldValue[] repro = GetFieldValues(svr, cookie, "Reproduced");
          foreach (CReportedByRecord rec in dft.reportedbylist)
          {
             CheckReportedByDropdownField(svr, cookie, "Reproduced", repro, rec.reproduced);
          }
          
          // Version Found
          CFieldValue[] versFnd = GetFieldValues(svr, cookie, "Version Found");
          foreach (CReportedByRecord rec in dft.reportedbylist)
          {
             CheckReportedByDropdownField(svr, cookie, "Version Found", versFnd, rec.foundinversion);
          }
          
          // Found By
          CFieldValue[] foundBy = GetFieldValues(svr, cookie, "Found by");
          foreach (CReportedByRecord rec in dft.reportedbylist)
          {
             if (rec.foundby != null)
             {
                if (!ArrayContainsValue(foundBy, rec.foundby))
                {
                   // Need to add a new local user
                   AddLocalUser(svr, cookie, rec.foundby);
                }
             }
          }
          
          // Test Config
          CFieldValue[] configs = GetFieldValues(svr, cookie, "Computer Config");
          // Test Configs come in two types, Standard or User. Standard is a regular CSystem or <not set>. User
          //   means use the config defined for the found by user
          foreach (CReportedByRecord rec in dft.reportedbylist)
          {
             if (rec.testconfigtype == (short)TestConfigType.eTestConfig_Standard)
             {
                if (rec.standardtestconfig != null)
                {
                   if (!ArrayContainsValue(configs, rec.standardtestconfig.systemname))
                   {
                      Console.WriteLine("Detected missing test config {0}, adding now", rec.standardtestconfig.systemname);
                      rec.standardtestconfig.recordid = svr.addTestConfig(cookie, rec.standardtestconfig);
                   }
                   else
                      rec.standardtestconfig.recordid = svr.getTestConfig(cookie, rec.standardtestconfig.systemname).recordid;
                }
             }
          }
       }

       private static void CheckInherentDropdownField(ttsoapcgi svr, long cookie, string fld, string value)
       {
          if (value != null)
          {
             if (!ArrayContainsValue(GetFieldValues(svr, cookie, fld), value))
                AddMissingValueToDefectField(svr, cookie, value, fld);
          }
       }
       
       private static void CheckReportedByDropdownField(ttsoapcgi svr, long cookie, string fld, 
                                                        CFieldValue[] values, string value)
       {
          if (value != null)
          {
             if (!ArrayContainsValue(values, value))
                AddMissingValueToDefectField(svr, cookie, value, fld);
          }
       }
       
       private static CFieldValue[] GetFieldValues(ttsoapcgi svr, long cookie, string fieldName)
       {
          CFieldValue[] values = null;
          try
          {
             values = svr.getDropdownFieldValuesForTable(cookie, "Defect", fieldName);
          }
          catch (SoapException e)
          {
             if (int.Parse(e.Detail.InnerText) == 16)
             {
                // This dropdown does not contain any values
                values = null;
             }
             else // Pass the exception on
                throw e;
          }
       
          return values;
       }
       
       private static bool ArrayContainsValue(CFieldValue[] values, string value)
       {
          bool bContains = false;
          if (values != null)
          {
             foreach (CFieldValue val in values)
             {
                if (val.value == value)
                {
                   bContains = true;
                   break;
                }
             }
          }
       
          return bContains;
       }
       
       private static void AddMissingValueToDefectField(ttsoapcgi svr, long cookie, string valueToAdd, string fieldName)
       {
          Console.WriteLine("Detected missing {0} {1}, adding now", fieldName, valueToAdd);
          CFieldValue[] addVals = new CFieldValue[1];
          addVals[0] = new CFieldValue();
          addVals[0].value = valueToAdd;
          svr.addDropdownFieldValuesForTable(cookie, "Defect", fieldName, addVals);
       }
       
       private static void AddLocalUser(ttsoapcgi svr, long cookie, string name)
       {
          Console.WriteLine("Detected missing user {0}, adding now", name);
          CUser usr = new CUser();
       
          // Name is in lastname, firstname format
          int idxOfComma = name.IndexOf(',');
          usr.lastname = name.Substring(0, idxOfComma);
          usr.firstname = name.Substring(idxOfComma + 2);
       
          usr.isactive = true;
          usr.isactiveSpecified = true;
          usr.isglobal = false;
          usr.isglobalSpecified = true;
          usr.notes = "Added by SOAP Replication";
       
          svr.addUser(cookie, usr);
       }

       private static void PopulateCustomFields(CDefect dft)
       {
          // Add multi-select dropdown
          CMultiSelectDropdownField codeArea = new CMultiSelectDropdownField();
          codeArea.name = "Code Area";
          // Array of values because this is a multi-select dropdown
          codeArea.values = new CFieldValue[3];
          codeArea.values[0] = new CFieldValue();
          codeArea.values[0].value = "Cache";
          codeArea.values[1] = new CFieldValue();
          codeArea.values[1].value = "Database";
          codeArea.values[2] = new CFieldValue();
          codeArea.values[2].value = "Transactions";
          
          // Add dropdown
          CDropdownField feature = new CDropdownField();
          feature.name = "Feature";
          // Only a single value
          feature.value = "List Window";
          
          // Add multi-line text data
          CStringField sourceInfo = new CStringField();
          sourceInfo.name = "Source Information";
          sourceInfo.value = "Copied from " + Environment.GetEnvironmentVariable("TTSTUDIOURL")
             + "\nProject: " + Environment.GetEnvironmentVariable("PROJNAME")
             + "\nCreated by " + dft.createdbyuser + " on " + dft.datetimecreated.ToLongDateString();
          
          // Add single line text data
          CStringField resource = new CStringField();
          resource.name = "Resource";
          resource.value = "3251-C23";
          
          // Add date field
          CDateField targetDate = new CDateField();
          targetDate.name = "Target Review Date";
          // Set a value
          targetDate.value = System.DateTime.Today.AddDays(7.0);
          // Un-set a value (use null on other platforms)
          //targetStart.value = System.DateTime.MinValue;
          // Ensure value specified is true or this value is ignored
          targetDate.valueSpecified = true;
          // .NET toolkit transforms System.DateTime into XML date format
          
          // Add boolean field
          CBooleanField virtualized = new CBooleanField();
          virtualized.name = "Can Be Virtualized";
          virtualized.value = true;
          
          // Add date+time field
          CDateTimeField deadline = new CDateTimeField();
          deadline.name = "Fix Deadline";
          // Set a value
          //deadline.value = System.DateTime.Now.AddHours(3.0);
          // Un-set a value (use null on other platforms)
          deadline.value = System.DateTime.MinValue;
          // Ensure value specified is true or this value is ignored
          deadline.valueSpecified = true;
          // .NET toolkit transforms System.DateTime into XML date time format
          
          // Add integer field
          CIntegerField storyPts = new CIntegerField();
          storyPts.name = "Story Points";
          storyPts.value = 8;
          
          // Add decimal field
          CDecimalField effort = new CDecimalField();
          effort.name = "Effort";
          effort.value = 20.5;
          
          // Add custom fields to the list
          dft.customFieldList = new CField[9] { codeArea, feature, sourceInfo, resource, targetDate, 
                                                virtualized, deadline, storyPts, effort };
       }
    }
}
